var CORSListener = (function() {
    var accessControlRequests = {};
        wtRequestsOrigins = {};

    var noRestriction = 'no_restriction';

    var responseRules = [{
        'data': {
            'name': 'Access-Control-Allow-Origin',
            'value': '*'
        },
        'mandatory': true,
        'fn': function(rule, header, details) {
            if (wtRequestsOrigins[details.requestId]) {
                if (header.value && header.value == '*') return;
                header.value = wtRequestsOrigins[details.requestId];
            }
        }
    }, {
        'data': {
            'name': 'Access-Control-Allow-Headers',
            'value': null
        },
        'mandatory': true,
        'fn': function(rule, header, details) {
            if (accessControlRequests[details.requestId] !== void 0) {
                header.value = accessControlRequests[details.requestId].headers;
            }
        }
    }, {
        'data': {
            'name': 'Access-Control-Allow-Credentials',
            'value': 'true'
        },
        'mandatory': false,
        'fn': null
    }, {
        'data': {
            'name': 'Access-Control-Allow-Methods',
            'value': 'POST, GET, OPTIONS, PUT, DELETE'
        },
        'mandatory': true,
        'fn': null
    }, {
        'data': {
            'name': 'Allow',
            'value': 'POST, GET, OPTIONS, PUT, DELETE'
        },
        'mandatory': true,
        'fn': null
    }];

    var updateCookie = function(cookie) {
        var cookieValues = cookie.split('; ');
        cookieValues.forEach(function(value) {
            if(value.includes('sameSite') && value !== 'sameSite=None') {
                value = 'sameSite=None';
            }
        });
        if(cookieValues.indexOf('sameSite=None') === -1) {
            cookieValues.push('sameSite=None');
        }
        if(cookieValues.indexOf('Secure') === -1) {
            cookieValues.push('Secure');
        }
        return cookieValues.join('; ')
    };

    var canEditCookie = function(tabId) {
        return storageManager.canEditCookie() || storageManager.isTabAllowedToFixSameSite(tabId);
    };
 
    /**
     * Update sameSite value to be "None" instead of "Lax"
     * Update secure value to be true
     * 
     * Note: Iframe doesn't have access to cookies which sameSite value is "Lax"
     * @param {*} details 
     */
    var setBrowserCookies = function(details) {
        var url = details.initiator,
            tabId = details.tabId;

        if(VE_ALLOWED_HOSTS.indexOf(url) === -1 && NWV_ALLOWED_HOSTS.indexOf(url) === -1 && canEditCookie(tabId)) {
            var hostname = new URL(url).hostname;
            var domain = hostname.replace('www.', '');
            chrome.cookies.getAll({
                url
            }, function (cookies) {
                cookies.forEach(cookie => {
                    if ((cookie.sameSite !== noRestriction || cookie.secure) && cookie.domain.indexOf(domain) !== -1) {
                        cookie.sameSite = noRestriction;
                        cookie.secure = true;
                        cookie.url = url;
                        delete cookie.hostOnly;
                        delete cookie.session;
                        chrome.cookies.remove({
                            url: url,
                            name: cookie.name
                        });
                        chrome.cookies.set(cookie);
                    }
                });
            })
        }
    };

    var updateCORSheaders = function(details) {
        responseRules.forEach(function(rule) {
            var flag = false;

            details.responseHeaders.forEach(function(header) {
                var data = rule.data;
                if (header.name.toLowerCase() === data.name.toLowerCase()) {
                    flag = true;
                    if (rule.fn) {
                        rule.fn.call(null, data, header, details);
                    } else {
                        if (rule.data.value) {
                            header.value = data.value;
                        }
                    }
                }
            });

            if (!flag && rule.mandatory) {
                if (rule.fn) {
                    rule.fn.call(null, rule.data, rule.data, details);
                }

                if (rule.data.value) {
                    details.responseHeaders.push(rule.data);
                }
            }
        });
        
        if(details.type === 'sub_frame') {
            details.responseHeaders.forEach(function(header) {
                if(header.name.toLowerCase() === 'set-cookie') {
                    header.value = updateCookie(header.value);
                }
            });
        }

        return {
            responseHeaders: details.responseHeaders
        };
    };

    var setWTRequestOrigins = function(requestId, url) {
        var pathArray = url.split('/'),
            origin = pathArray[0] + '//' + pathArray[2];
        if (!wtRequestsOrigins[requestId]) {
            wtRequestsOrigins[requestId] = origin;
        }
    };

    var setCORSdata = function(details) {
        var headers = details.requestHeaders,
            requestId = details.requestId;
        headers.forEach(function(header) {
            var headerName = header.name.toLowerCase();
            switch (headerName) {
                case 'origin':
                    var url = header.value.toLowerCase();
                    if (url.indexOf('webtrends') !== -1) {
                        wtRequestsOrigins[requestId] = header.value;
                    }
                    break;
                case 'referer':
                    var url = header.value.toLowerCase();
                    if (url.indexOf('webtrends') !== -1) {
                        if (url.indexOf('htmlfile') !== -1) {
                            setWTRequestOrigins(requestId, url);
                        }
                    }
                    break;
                case 'access-control-request-headers':
                    if (!accessControlRequests[requestId]) {
                        accessControlRequests[requestId] = {};
                    }
                    accessControlRequests[requestId].headers = header.value;
                    break;
            }
        });

    };

    var start = function(tabId) {
        var filter = {
            urls: ['*://*/*'],
            tabId: tabId
        };

        chrome.webRequest.onBeforeSendHeaders.addListener(
            setBrowserCookies, filter, ['blocking', 'requestHeaders']
        );

        chrome.webRequest.onBeforeSendHeaders.addListener(
            setCORSdata, filter, ['blocking', 'requestHeaders']
        );

        chrome.webRequest.onHeadersReceived.addListener(
            updateCORSheaders, filter, ['blocking', 'responseHeaders', 'extraHeaders']
        );
    };

    var stop = function() {
        chrome.webRequest.onBeforeSendHeaders.removeListener(setCORSdata);
        chrome.webRequest.onBeforeSendHeaders.removeListener(setBrowserCookies);
        chrome.webRequest.onHeadersReceived.removeListener(updateCORSheaders);
    };

    return {
        start: start,
        stop: stop
    };
})();

optimizeListener.register(CORSListener);
