/**
 * This class is responsible for handling all the actions done in Visual Editor
 * and NVW tab to ensure any page can successfully be opened in it by adding
 * corresponding listeners.
 */

const VE_ALLOWED_HOSTS = [
    'https://app.webtrends-optimize.com'
];

const NWV_ALLOWED_HOSTS = [
    'https://app.webtrends-optimize.com'
];

const isURLInHosts = function(url, hosts) {
    var isContained = false;
    for (var i = 0, len = hosts.length; i < len; ++i) {
        if (url.indexOf(hosts[i]) !== -1) {
            isContained = true;
            break;
        }
    }
    return isContained;
};

var optimizeListener = (function() {
    var listeners = [];

    if (!chrome.cookies) {
        chrome.cookies = chrome.experimental.cookies;
    }

    var stopListeners = function() {
        for (var i = 0, len = listeners.length; i < len; ++i) {
            listeners[i].stop();
        }
    };

    var startListeners = function(tabId) {
        for (var i = 0, len = listeners.length; i < len; ++i) {
            listeners[i].start(tabId);
        }
    };

    var checkListerners = function(tabId) {
        if (tabManager.getWTTab(tabId)) {
            tabManager.deleteWTTab(tabId);
            if (!tabManager.hasWTTab()) {
                stopListeners();
            }
        }
    };

    var register = function(listener) {
        listeners.push(listener);
    };

    chrome.tabs.onUpdated.addListener(function(tabId, changeInfo, tab) {
        if (storageManager.isToolEnabled()) {
            var url = tab.url.toLowerCase();
            if (isURLInHosts(url, VE_ALLOWED_HOSTS) || isURLInHosts(url, NWV_ALLOWED_HOSTS)) {
                if (tabManager.getWTTab(tabId)) {
                    return;
                }
                startListeners(tabId);
                tabManager.addWTTab(tabId);
            } else {
                checkListerners(tabId);
            }
        } else {
            checkListerners(tabId);
        }
    });

    chrome.tabs.onRemoved.addListener(function(tabId, info) {
        checkListerners(tabId);
    });

    chrome.tabs.onReplaced.addListener(function(addedTabId, removedTabId) {
        checkListerners(removedTabId);
    });

    return {
        register: register
    };
})();
