var storageManager = storageManager || (function() {
    var toolEnabled = false,
        canEditContent = false,
        canEditCookie = false,
        counter = 3,
        cbs = [],
        allowedTabsToFixSameSite = {};

    var register = function(fn) {
        cbs.push(fn);
        check();
    };

    var addTabToAllowFixSameSitList = function(tabId, canEdit) {
        if(canEdit) {
            allowedTabsToFixSameSite[tabId] = canEdit;
        } else if(allowedTabsToFixSameSite[tabId]){
            delete allowedTabsToFixSameSite[tabId];
        }
    };

    var check = function() {
        if (counter === 0) {
            for (var i = 0, len = cbs.length; i < len; ++i) {
               if (typeof cbs[i] === 'function') {
                   cbs[i]();
               }
            }
        }
    };

    var isToolEnabled = function() {
        return toolEnabled;
    };

    var isTabAllowedToFixSameSite = function(tabId) {
        return allowedTabsToFixSameSite[tabId];
    };

    var canEditContent = function() {
        return canEditContent;
    };

    var canEditCookie = function() {
        return canEditCookie;
    };

    chrome.storage.local.get('enable_editing_tools', function(result) {
        var value = result['enable_editing_tools'];
        if (!value || (value && value === 'on')) {
            toolEnabled = true;
        } else {
            toolEnabled = false;
        }
        --counter;
        check();
    });

    chrome.storage.local.get('allow_editing_javascript', function(result) {
        var value = result['allow_editing_javascript'];
        if (value && value === 'on') {
            canEditContent = true;
        } else {
            canEditContent = false;
        }
        --counter;
        check();
    });

    chrome.storage.local.get('allow_editing_cookie', function(result) {
        var value = result['allow_editing_cookie'];
        if (value && value === 'on') {
            canEditCookie = true;
        } else {
            canEditCookie = false;
        }
        --counter;
        check();
    });

    chrome.storage.onChanged.addListener(function(changes, areaName) {
        if (changes.hasOwnProperty('enable_editing_tools')) {
            if (changes.enable_editing_tools.newValue === 'on') {
                toolEnabled = true;
            } else {
                toolEnabled = false;
            }
        }
        if (changes.hasOwnProperty('allow_editing_javascript')) {
            if (changes.allow_editing_javascript.newValue === 'on') {
                canEditContent = true;
            } else {
                canEditContent = false;
            }
        }
        if (changes.hasOwnProperty('allow_editing_cookie')) {
            if (changes.allow_editing_cookie.newValue === 'on') {
                canEditCookie = true;
            } else {
                canEditCookie = false;
            }
        }
    });

    return {
        isToolEnabled: isToolEnabled,
        canEditContent: canEditContent,
        canEditCookie: canEditCookie,
        register: register,
        addTabToAllowFixSameSitList: addTabToAllowFixSameSitList,
        isTabAllowedToFixSameSite: isTabAllowedToFixSameSite
    };
})();
