var scriptHandler = (function() {

    var pageScripts = [];

    var replaceContent = function(cleanDocument, regexes, data, output) {
        for (var i = 0, len = regexes.length; i < len; ++i) {
            cleanDocument = cleanDocument.replace(regexes[i], function(token) {
                output.replaced = true; return data;
            });
        }
        return cleanDocument;
    };

    var clearIframeBreakingCodes = function(scriptText) {
        var replaceExpressions = {
                'window.top': 'window.self',
                'top.location': 'self.location',
                'document.top': 'document',
                'top.document': 'document',
                'top.window': 'window.self',
                'window.parent.location': 'window.location',
                'document.write': 'document.wt_appendHtml'
            },
            cleanDocument = scriptText,
            output = {
               script: '',
               replaced: false
            },
            regexes = [/top\s?=?==\s?self/, /self\s?=?==\s?top/];

        replaceContent(cleanDocument, regexes, 'true', output);
        regexes = [/top\s?!=?=\s?self/, /self\s?!=?=\s?top/];
        replaceContent(cleanDocument, regexes, 'false', output);

        for (var key in replaceExpressions) {
            cleanDocument = cleanDocument.replace(
                new RegExp(key, 'g'),
                function(token) {
                    output.replaced = true; return replaceExpressions[key];
                }
            );
        }
        if (output.replaced) {
            // In the some cases after breaking codes clean-up the dom is already loaded
            // $(document).ready is executed immediately which can cause an error
            // Moving $(document).ready to call inside setTimeout to avoid such issues
            cleanDocument = cleanDocument.replace(/\$\(document\).ready/g, 'document.wt_ready');
        }
        output.script = cleanDocument;
        return output;
    };

    /* In some cases the parent reference of the observed node is broken as
     * actual dom element and as a result the is no effect to add
     * the script in the parent element (script will not appear in the actual
     * dom tree).
     * This function is checking for the described cases.
     */
    var isElementInDocument = function(element) {
        var parentElement = element.parentElement;
        if (parentElement) {
            return isElementInDocument(parentElement);
        } else if (!parentElement && element.nodeName.toLowerCase() === 'html') {
            return true;
        }
        return false;
    };

    var addScript = function(content, attributes, id, parent, src, type) {
        var element = document.createElement('script');
        element.setAttribute('type', type);
        element.setAttribute('id', id);
        element.setAttribute('is_wt_updated', 'true');

        if (src != '') {
            element.setAttribute('src', src);
        }
        if (content !== '') {
            element.appendChild(document.createTextNode(content));
        }
        if (parent && isElementInDocument(parent)) {
            parent.appendChild(element);
        } else {
            document.getElementsByTagName('head')[0].appendChild(element);
        }

    };

    var recoverScripts = function() {
        var i = 0;
        while (pageScripts.length > 0 && pageScripts[i].isReady) {
            addScript(pageScripts[i].content,
                    pageScripts[i].attributes,
                    pageScripts[i].id,
                    pageScripts[i].parent,
                    pageScripts[i].src,
                    pageScripts[i].type);
            pageScripts.shift();
        }
    };

    var reconstructScript = function(script, scriptNode, content, url) {
        if (script.parent) {
            try {
                script.parent.removeChild(scriptNode);
            } catch (e) {
                utils.handleError(e);
            }
        }
        script.content = content;
        script.attributes = scriptNode.attributes;
        if (url) {
            script.src = url;
        }
        script.isReady = true;
        recoverScripts();
    };

    var getScriptPrototype = function(scriptNode) {
        var script = {
            content: '',
            attributes: '',
            id: '',
            src: '',
            parent: scriptNode.parentNode || scriptNode.parentElement,
            type: 'text/javascript',
            isReady: false,
            uid: Math.random()
        };

        if (scriptNode.getAttribute('id')) {
            script.id = scriptNode.getAttribute('id');
        }

        if (scriptNode.getAttribute('type')) {
            script.type = scriptNode.getAttribute('type');
        }

        return script;
    };

    var clearScript = function(scriptNode, url) {
        var script = getScriptPrototype(scriptNode),
            oldContent,
            tryCount,
            checkExist,
            content;

        scriptNode.setAttribute('wt-uid', script.uid);
        if (url) {
            if (!script.id) {
                script.id = url;
            }
            pageScripts.push(script);
            oldContent = scriptNode.innerHTML;
            scriptNode.removeAttribute('src');
            scriptNode.innerHTML = '';
            utils.getResource(url).then(function(result) {
                var newContent = clearIframeBreakingCodes(result),
                    attributes;
                if (newContent.replaced) {
                    attributes = scriptNode.attributes;
                    if (script.parent) {
                        try {
                            var node = document.querySelector('[wt-uid="' + script.uid +'"]');
                            var parentElement = script.parent.contains(scriptNode) ? script.parent : node.parentElement;
                            parentElement.removeChild(scriptNode);
                        } catch (e) {
                            utils.handleError(e);
                        }
                    }
                    script.content = newContent.script;
                    script.attributes = attributes;
                    script.isReady = true;
                    recoverScripts();
                } else {
                    reconstructScript(script, scriptNode, oldContent, url);
                }
            }).catch (function(e) {
                utils.handleError(e);
                reconstructScript(script, scriptNode, oldContent, url);
            });
        } else {
            content = scriptNode.innerHTML;
            if (content !== '' && content !== null) {
                content = clearIframeBreakingCodes(content).script;
                pageScripts.push(script);
                reconstructScript(script, scriptNode, content);
            } else {
                tryCount = 0;
                pageScripts.push(script);
                checkExist = setInterval(function() {
                    var content = scriptNode.innerHTML;
                    if (content != '' && content != null) {
                        content = clearIframeBreakingCodes(content).script;
                        reconstructScript(script, scriptNode, content);
                        clearInterval(checkExist);
                    }
                    if (tryCount > 20) {
                        clearInterval(checkExist);
                    }
                    ++tryCount;
                }, 100);
            }
        }
    };

    return {
       clearScript: clearScript
    };
})();
