var ScriptsObserver = (function() {
    const JS_LIBS = ['require', 'angular', 'ng-', 'moment',
          'jquery', 'bootstrap', 'backbone', 'react', 'underscore',
          'ext', 'sencha', 'cappuchino', 'chaplin', 'echo', 'ember',
          'enyo', 'knockout', 'meteor', 'mojito', 'moo', 'next',
          'prototype', 'rialto', 'modernizr', 'utag'];

    var isJSLibrary = function(url) {
        for (var i = 0, len = JS_LIBS.length; i < len; ++i) {
            if (url.indexOf(JS_LIBS[i]) !== -1) {
                return true;
            }
        }
        return false;
    };

    var handleTag = function(tag) {
        var url;
        if (tag.nodeName.toLowerCase() === 'script') {
            if (tag.attributes.is_wt_updated) {
                return;
            }
            url = tag.getAttribute('src');
            if (url && isJSLibrary(url)) {
                return;
            }
            scriptHandler.clearScript(tag, url);
        }
    };

    var handleNode = function(node) {
        try {
            if (node.nodeName.toLowerCase() === 'head') {
                for (var j = 0, len = node.childNodes.length; j < len; ++j) {
                    handleTag(node.childNodes[j]);
                }
            } else {
                handleTag(node);
            }
        } catch (e) {
            utils.handleError(e);
       }
    };

    var workroundDocumentWrite = function() {
        var content =
            'document.wt_appendHtml = function(html) {' +
                'if (document.body) {' +
                    'document.body.innerHTML += html;' +
                '} else {' +
                    'document.getElementsByTagName(' +
                        '"head")[0].innerHTML += html;' +
                '}' +
            '};' +
            'document.wt_ready = function(functionBody) {' +
                'setTimeout(function(){$(document).ready(functionBody)}, 0);' +
            '}'
            ;
        utils.addElement(content, 'javascript', 'script', 'wt_custom_script');
    };

    var observe = function() {
        var contentObserver = new MutationObserver(function(mutations) {
            var len;
            mutations.forEach(function(mutation) {
                if (!mutation.addedNodes) return;
                if (document.getElementById('wt_custom_script') === null) {
                    workroundDocumentWrite();
                }
                len = mutation.addedNodes.length;
                for (var i = 0; i < len; ++i) {
                    handleNode(mutation.addedNodes[i]);
                }
            });
        });

        contentObserver.observe(document, {
            childList: true,
            subtree: true,
            attributes: false,
            characterData: false
        });
    };

    var init = function() {
        if (storageManager.isToolEnabled() && storageManager.canEditContent() &&
            utils.inIframe()) {

            observe();
        }
    };
    storageManager.register(init);
})();
